@@ -143,14 +143,17 @@ class FileBytes:
             raise Exception("Another FileBytes instance representing the same file was written back!")
 
         # Add data to the end of our representation.
-        for off, change in enumerate(data[:]):
-            loc = self.__patchlength + off
-            self.__patches[loc] = change
-            self.__lowest_patch = min(self.__lowest_patch, loc) if self.__lowest_patch is not None else loc
-            self.__highest_patch = max(self.__highest_patch, loc + 1) if self.__highest_patch is not None else (loc + 1)
-            self.__regions.clear()
+        data = data[:]
+        lowest_loc = self.__patchlength
+        highest_loc = self.__patchlength + len(data)
 
-        self.__patchlength += len(data)
+        for off, change in enumerate(data):
+            self.__patches[self.__patchlength + off] = change
+
+        self.__lowest_patch = min(self.__lowest_patch, lowest_loc) if self.__lowest_patch is not None else lowest_loc
+        self.__highest_patch = max(self.__highest_patch, highest_loc + 1) if self.__highest_patch is not None else (highest_loc + 1)
+        self.__regions.clear()
+        self.__patchlength = highest_loc
 
     def truncate(self, size: int) -> None:
         if self.__unsafe:
@@ -283,7 +286,7 @@ class FileBytes:
 
         # Determine start of slice
         if key.start is None:
-            start = 0 if step > 0 else self.__patchlength
+            start = 0 if step > 0 else (self.__patchlength - 1)
         elif key.start < 0:
             start = self.__patchlength + key.start
         else:
@@ -404,7 +407,7 @@ class FileBytes:
 
                         # Append any amount of data we need to read past the end of the file.
                         if len(data) < stop - start:
-                            data.extend([0] * (stop - len(data)))
+                            data.extend([0] * ((stop - start) - len(data)))
                     else:
                         data = [0] * (stop - start)
 
@@ -415,23 +418,25 @@ class FileBytes:
 
                     return bytes(data)
             elif start > stop and step == -1:
+                start += 1
+                stop += 1
                 if not modifications:
                     # This is just a continguous read, reversed
-                    self.__handle.seek(stop + 1)
+                    self.__handle.seek(stop)
                     return self.__handle.read(start - stop)[::-1]
                 else:
-                    if (stop + 1) < self.__filelength:
-                        self.__handle.seek(stop + 1)
+                    if stop < self.__filelength:
+                        self.__handle.seek(stop)
                         data = [x for x in self.__handle.read(start - stop)]
 
                         # Append any amount of data we need to read past the end of the file.
                         if len(data) < start - stop:
-                            data.extend([0] * (start - len(data)))
+                            data.extend([0] * ((start - stop) - len(data)))
                     else:
                         data = [0] * (start - stop)
 
                     # Now we have to modify the data with our own overlay.
-                    for index, off in enumerate(range(stop + 1, start + 1)):
+                    for index, off in enumerate(range(stop, start)):
                         if off in self.__patches:
                             data[index] = self.__patches[off]
 
